"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useCssAndCx = exports.cssAndCxFactory = void 0;
const classnames_1 = require("./tools/classnames");
const serialize_1 = require("@emotion/serialize");
const utils_1 = require("@emotion/utils");
const useGuaranteedMemo_1 = require("./tools/useGuaranteedMemo");
const cache_1 = require("./cache");
const refPropertyName = "ref";
exports.cssAndCxFactory = (() => {
    function merge(registered, css, className) {
        const registeredStyles = [];
        const rawClassName = (0, utils_1.getRegisteredStyles)(registered, registeredStyles, className);
        if (registeredStyles.length < 2) {
            return className;
        }
        return rawClassName + css(registeredStyles);
    }
    function cssAndCxFactory(params) {
        const { cache } = params;
        const css = (...args) => {
            let ref = undefined;
            scope: {
                if (args.length !== 1) {
                    break scope;
                }
                const [arg] = args;
                if (!(arg instanceof Object)) {
                    break scope;
                }
                if (!(refPropertyName in arg)) {
                    break scope;
                }
                ref = arg[refPropertyName];
                const argCopy = Object.assign({}, arg);
                delete argCopy[refPropertyName];
                args = [argCopy];
            }
            const serialized = (0, serialize_1.serializeStyles)(args, cache.registered);
            (0, utils_1.insertStyles)(cache, serialized, false);
            return `${cache.key}-${serialized.name}${ref === undefined ? "" : ` ${ref}`}`;
        };
        const cx = (...args) => merge(cache.registered, css, (0, classnames_1.classnames)(args));
        return { css, cx };
    }
    return { cssAndCxFactory };
})().cssAndCxFactory;
/** Will pickup the contextual cache if any */
function useCssAndCx() {
    const cache = (0, cache_1.useTssEmotionCache)();
    const { css, cx } = (0, useGuaranteedMemo_1.useGuaranteedMemo)(() => (0, exports.cssAndCxFactory)({ cache }), [cache]);
    return { css, cx };
}
exports.useCssAndCx = useCssAndCx;
